@@ -1,6 +1,6 @@
 from mindsdb_sql.parser.ast.base import ASTNode
 from mindsdb_sql.parser.utils import indent
-
+from mindsdb_sql.parser.ast.select.identifier import Identifier
 
 class CreateView(ASTNode):
     def __init__(self,
@@ -9,6 +9,9 @@ class CreateView(ASTNode):
                  from_table=None,
                  *args, **kwargs):
         super().__init__(*args, **kwargs)
+        # todo remove it
+        if isinstance(name, Identifier):
+            name = name.parts[-1]
         self.name = name
         self.query_str = query_str
         self.from_table = from_table
@@ -17,6 +20,7 @@ class CreateView(ASTNode):
         ind = indent(level)
         ind1 = indent(level+1)
         name_str = f'\n{ind1}name={repr(self.name)},'
+        # name_str = f'\n{ind1}name={self.name.to_string()},'
         from_table_str = f'\n{ind1}from_table=\n{self.from_table.to_tree(level=level+2)},' if self.from_table else ''
         query_str = f'\n{ind1}query="{self.query_str}"'
 
@@ -29,6 +33,8 @@ class CreateView(ASTNode):
 
     def get_string(self, *args, **kwargs):
         from_str = f'FROM {str(self.from_table)} ' if self.from_table else ''
+        # out_str = f'CREATE VIEW {self.name.to_string()} {from_str}AS ( {self.query_str} )'
         out_str = f'CREATE VIEW {str(self.name)} {from_str}AS ( {self.query_str} )'
 
         return out_str
+
