@@ -25,6 +25,7 @@ class MindsDBLexer(Lexer):
 
         PREDICTOR, PREDICTORS, DATASOURCE, INTEGRATION, INTEGRATIONS,DATASOURCES,
         STREAM, STREAMS, PUBLICATION, PUBLICATIONS, VIEW, VIEWS, DATASETS, DATASET,
+        MODEL, MODELS, ML_ENGINE, ML_ENGINES, HANDLERS,
 
         LATEST, HORIZON, USING,
         ENGINE, TRAIN, PREDICT, PARAMETERS,
@@ -97,6 +98,11 @@ class MindsDBLexer(Lexer):
     DATASETS = r'\bDATASETS\b'
     DATASET = r'\bDATASET\b'
     LATEST = r'\bLATEST\b'
+    MODEL = r'\bMODEL\b'
+    MODELS = r'\bMODELS\b'
+    ML_ENGINE = r'\bML_ENGINE\b'
+    ML_ENGINES = r'\bML_ENGINES\b'
+    HANDLERS = r'\bHANDLERS\b'
 
     # Misc
     SET = r'\bSET\b'
